@@ -1,33 +1,21 @@
 // -*- c++ -*-
 //
-// Copyright 1997, 1998, 1999 University of Notre Dame.
+// Software License for MTL
+// 
+// Copyright (c) 2001-2005 The Trustees of Indiana University. All rights reserved.
+// Copyright (c) 1998-2001 University of Notre Dame. All rights reserved.
 // Authors: Andrew Lumsdaine, Jeremy G. Siek, Lie-Quan Lee
-//
+// 
 // This file is part of the Matrix Template Library
-//
-// You should have received a copy of the License Agreement for the
-// Matrix Template Library along with the software;  see the
-// file LICENSE.  If not, contact Office of Research, University of Notre
-// Dame, Notre Dame, IN  46556.
-//
-// Permission to modify the code and to distribute modified code is
-// granted, provided the text of this NOTICE is retained, a notice that
-// the code was modified is included with the above COPYRIGHT NOTICE and
-// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
-// file is distributed with the modified code.
-//
-// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
-// By way of example, but not limitation, Licensor MAKES NO
-// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
-// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
-// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
-// OR OTHER RIGHTS.
-//
+// 
+// See also license.mtl.txt in the distribution.
 //===========================================================================
 
 #ifndef _MTL_MTL_H_
 #define _MTL_MTL_H_
 
+#include "mtl/mtl_decl.h" 
+
 #include <functional>
 #include <iostream>
 #include "mtl/mtl_limits.h"
@@ -61,7 +49,7 @@
     not be passed by value for the output argument
     since they are not handles.
  */
-#define MTL_OUT(X) const X&
+// #define MTL_OUT(X) const X& // moved to mtl_decl.h 
 
 namespace mtl {
 
@@ -114,7 +102,7 @@ template <class Vector> inline
 typename linalg_traits<Vector>::value_type
 sum(const Vector& x)
 {
-  return sum__(x, dim_n<Vector>::RET());
+  return sum__(x, typename dim_n<Vector>::RET());
 }
 
 #include "mtl/mtl_set.h"
@@ -151,7 +139,7 @@ template <class Vector, class T> inline
 void
 scale_dim(Vector& x, const T& alpha, oned_tag)
 {
-  oned_scale(x, alpha, dim_n<Vector>::RET());
+  oned_scale(x, alpha, typename dim_n<Vector>::RET());
 }
 
 template <class Matrix, class T>
@@ -245,7 +233,7 @@ template <class Vector>
 inline typename linalg_traits<Vector>::magnitude_type
 one_norm(const Vector& x, oned_tag)
 {
-  return oned_one_norm(x, dim_n<Vector>::RET());
+  return oned_one_norm(x, typename dim_n<Vector>::RET());
 }
 
 
@@ -264,8 +252,7 @@ oned_two_norm(const Vector& x, fast::count<0>)
 {
   typedef typename Vector::value_type T;
   typedef typename number_traits<T>::magnitude_type M;
-  using std::sqrt;
-  return ::sqrt(mtl_algo::accumulate(x.begin(), x.end(), M(), sqr_add()));
+  return std::sqrt(mtl_algo::accumulate(x.begin(), x.end(), M(), sqr_add()));
 }
 
 #if USE_BLAIS
@@ -275,8 +262,7 @@ oned_two_norm(const Vector& x, fast::count<N>)
 {
   typedef typename Vector::value_type T;
   typedef typename number_traits<T>::magnitude_type M;
-  using std::sqrt;
-  return ::sqrt(fast::accumulate(x.begin(), fast::count<N>(), M(), sqr_add()));
+  return std::sqrt(fast::accumulate(x.begin(), fast::count<N>(), M(), sqr_add()));
 }
 #endif
 
@@ -297,7 +283,7 @@ template <class Vector>
 inline typename linalg_traits<Vector>::magnitude_type
 two_norm(const Vector& x)
 {
-  return oned_two_norm(x, dim_n<Vector>::RET());
+  return oned_two_norm(x, typename dim_n<Vector>::RET());
 }
 
 //: add square
@@ -821,7 +807,7 @@ public:
     if (scal != T(0)) {
       T a_scl = a_in / scal;
       T b_scl = b_in / scal;
-      r = scal * sqrt(a_scl * a_scl + b_scl * b_scl);
+      r = scal * std::sqrt(a_scl * a_scl + b_scl * b_scl);
       if (roe < T(0)) r *= -1;
       c_ = a_in / r;
       s_ = b_in / r;
@@ -854,14 +840,14 @@ public:
       if (abs_a > abs_b) {
         // 1/cs = sqrt( 1 + |b|^2 / |a|^2 )
         T t = abs_b / abs_a;
-        T tt = sqrt(T(1) + t * t);
+        T tt = std::sqrt(T(1) + t * t);
         c_ = T(1) / tt;
         s_ = t * c_;
         r_ = a * tt;
       } else {
         // 1/sn = sign(a) * sqrt( 1 + |a|^2/|b|^2 )
         T t = abs_a / abs_b;
-        T tt = sqrt(T(1) + t * t);
+        T tt = std::sqrt(T(1) + t * t);
         s_ = sign(a) / tt;
         c_ = t * s_;
         r_ = b * tt;
@@ -975,18 +961,18 @@ public:
         g2 = abs_sq(gs);
         fs = f / f1;
         f2 = abs_sq(fs);
-        d = sqrt(T(1) + g2 / f2);
+        d = std::sqrt(T(1) + g2 / f2);
         cs = T(1) / d;
         sn = MTL_CONJ(gs) * fs * (cs / f2);
         r_ = f * d;
       } else {
         fs = f / g1;
         f2 = abs_sq(fs);
-        fa = sqrt(f2);
+        fa = std::sqrt(f2);
         gs = g / g1;
         g2 = abs_sq(gs);
-        ga = sqrt(g2);
-        d = sqrt(T(1) + f2 / g2);
+        ga = std::sqrt(g2);
+        d = std::sqrt(T(1) + f2 / g2);
         di = T(1) / d;
         cs = (fa / ga ) * di;
         ss = (MTL_CONJ(gs) * fs) / (fa * ga);
@@ -1092,18 +1078,18 @@ public:
         g2 = abs_sq(gs);
         fs = f / f1;
         f2 = abs_sq(fs);
-        d = sqrt(T(1) + g2 / f2);
+        d = std::sqrt(T(1) + g2 / f2);
         cs = T(1) / d;
         sn = MTL_CONJ(gs) * fs * (cs / f2);
         r_ = f * d;
       } else {
         fs = f / g1;
         f2 = abs_sq(fs);
-        fa = sqrt(f2);
+        fa = std::sqrt(f2);
         gs = g / g1;
         g2 = abs_sq(gs);
-        ga = sqrt(g2);
-        d = sqrt(T(1) + f2 / g2);
+        ga = std::sqrt(g2);
+        d = std::sqrt(T(1) + f2 / g2);
         di = T(1) / d;
         cs = (fa / ga ) * di;
         ss = (MTL_CONJ(gs) * fs) / (fa * ga);
@@ -1202,18 +1188,18 @@ public:
         g2 = abs_sq(gs);
         fs = f / f1;
         f2 = abs_sq(fs);
-        d = sqrt(T(1) + g2 / f2);
+        d = std::sqrt(T(1) + g2 / f2);
         cs = T(1) / d;
         sn = MTL_CONJ(gs) * fs * (cs / f2);
         r_ = f * d;
       } else {
         fs = f / g1;
         f2 = abs_sq(fs);
-        fa = sqrt(f2);
+        fa = std::sqrt(f2);
         gs = g / g1;
         g2 = abs_sq(gs);
-        ga = sqrt(g2);
-        d = sqrt(T(1) + f2 / g2);
+        ga = std::sqrt(g2);
+        d = std::sqrt(T(1) + f2 / g2);
         di = T(1) / d;
         cs = (fa / ga ) * di;
         ss = (MTL_CONJ(gs) * fs) / (fa * ga);
@@ -1279,7 +1265,7 @@ class modified_givens {
 
 template <class T>
 inline T two_norm3(const T& x, const T& y, const T& z) {
-  return sqrt(x*x + y*y + z*z);
+  return std::sqrt(x*x + y*y + z*z);
 }
 
 //: Generate Householder Transform
@@ -2445,7 +2431,8 @@ inline void
 oned_copy(const VecX& x, VecY& y, dense_tag, dense_tag) MTL_THROW_ASSERTION
 {
   MTL_ASSERT(x.size() <= y.size(), "mtl::copy()");
-  copy__(x, y, dim_n<VecX>::RET());
+  // copy__(x, y, dim_n<VecX>::RET()); gcc4
+  copy__(x, y, typename dim_n<VecX>::RET());
 }  
 
 #if 0
@@ -2713,7 +2700,7 @@ add__(const VecX& x, VecY& y, oned_tag) MTL_THROW_ASSERTION
 {
   MTL_ASSERT(x.size() <= y.size(), "mtl::add()");
 
-  add__(x, y, dim_n<VecX>::RET());
+  add__(x, y, typename dim_n<VecX>::RET());
 }
 
 
@@ -2740,7 +2727,8 @@ oned_add(const VecX& x, const VecY& y, VecZ& z_, sparse_tag)
 {
   
   typedef typename VecZ::value_type T;
-  compressed1D<T> z;
+  compressed1D<T> z(z_.size());
+  // compressed1D<T> z; push_back doesn't change size and get an error - PG
   typedef typename VecX::const_iterator xiter;
   typedef typename VecY::const_iterator yiter;
   
@@ -2777,7 +2765,8 @@ template <class VecX, class VecY, class VecZ>
 inline void
 oned_add(const VecX& x, const VecY& y, VecZ& z, dense_tag) MTL_THROW_ASSERTION
 {
-  oned_add(x, y, z, dim_n<VecX>::RET());
+  // oned_add(x, y, z, dim_n<VecX>::RET()); gcc4
+  oned_add(x, y, z, typename dim_n<VecX>::RET());
 }
 
 
@@ -3003,7 +2992,7 @@ ele_mult(const VecX& x, const VecY& y, MTL_OUT(VecZ) z_) MTL_THROW_ASSERTION
   MTL_ASSERT(x.size() <= y.size(), "mtl::ele_mult()");
   MTL_ASSERT(x.size() <= z.size(), "mtl::ele_mult()");
 
-  ele_mult(x, y, z, dim_n<VecX>::RET());
+  ele_mult(x, y, z, typename dim_n<VecX>::RET());
 }
 
 
@@ -3080,7 +3069,7 @@ inline void
 swap(VecX& x, VecY& y, oned_tag) MTL_THROW_ASSERTION
 {
   MTL_ASSERT(x.size() <= y.size(), "mtl::swap()");
-  swap(x, y, dim_n<VecX>::RET());
+  swap(x, y, typename dim_n<VecX>::RET());
 }  
 
 
@@ -3153,7 +3142,7 @@ template <class VecX, class VecY, class T>
 inline T
 dot(const VecX& x, const VecY& y, T s, dense_tag, dense_tag)
 {
-  return dot(x, y, s, dim_n<VecX>::RET());
+  return dot(x, y, s, typename dim_n<VecX>::RET());
 }
 
 template <class InputIterator1, class InputIterator2, class T>
@@ -3305,7 +3294,7 @@ inline T
 dot_conj(const VecX& x, const VecY& y, T s) MTL_THROW_ASSERTION
 {
   MTL_ASSERT(x.size() <= y.size(), "mtl::dot_conj()");
-  return dot_conj(x, y, s, dim_n<VecX>::RET());
+  return dot_conj(x, y, s, typename dim_n<VecX>::RET());
 }
 
 //: Dot Conjugate:   <tt>s <- x . conj(y)</tt>
